package sample;

import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import jp.co.intra_mart.foundation.bloommaker.exception.BloomMakerException;
import jp.co.intra_mart.foundation.bloommaker.exception.BloomMakerPreprocessException;
import jp.co.intra_mart.foundation.bloommaker.route.preprocess.BMContentPreprocessor;
import jp.co.intra_mart.foundation.bloommaker.route.preprocess.BMContentPreprocessorContext;
import jp.co.intra_mart.foundation.context.Contexts;
import jp.co.intra_mart.foundation.context.model.AccountContext;
import jp.co.intra_mart.foundation.i18n.datetime.format.DateTimeFormatSetInfo;
import jp.co.intra_mart.foundation.i18n.datetime.format.DateTimeFormatter;
import jp.co.intra_mart.foundation.i18n.datetime.format.SystemDateTimeFormat;

/**
 * IM-BloomMaker の前処理クラスのサンプルです。
 * @author INTRAMART
 * @version 8.0.2
 * @since 8.0.0
 */
public class PreProcessor implements BMContentPreprocessor {

    @Override
    public Map<String, Object> execute(final BMContentPreprocessorContext context) throws BloomMakerException {
        // 返却するマップ
        final Map<String, Object> result = new HashMap<>();

        // 単純なキー・バリューをセットします。
        result.put("foo", "bar");

        // アカウントコンテキストをセットします。
        final Map<String, Object> accountContextMap = new HashMap<>();
        final AccountContext accountContext = Contexts.get(AccountContext.class);
        accountContextMap.put("calendarId", accountContext.getCalendarId());
        accountContextMap.put("encoding", accountContext.getEncoding());
        accountContextMap.put("userCd", accountContext.getUserCd());
        result.put("accountContext", accountContextMap);

        // リクエストパラメータを取得します。
        final String targetLocale = (String) context.getParsedRequestParameters().get("locale");

        // 取得したロケールに応じたフォーマットで現在日時をフォーマットします。
        final DateTimeFormatSetInfo[] formats = SystemDateTimeFormat.getFormatSets();
        final String formatsetId = Arrays.asList(formats).stream().filter(format -> format.getLocale().toString().equals(targetLocale)).findFirst().map(format -> format.getFormatSetId()).orElse(SystemDateTimeFormat.getDefaultFormatSet().getFormatSetId());
        final String format = SystemDateTimeFormat.getFormats(formatsetId).get("IM_DATETIME_FORMAT_DATE_STANDARD");
        final DateTimeFormatter formatter = DateTimeFormatter.withPattern(format);
        result.put("currentDate", formatter.format(new Date()));

        // 何らかのエラーが発生したことを示すフラグ。
        // 本来は API の返り値を格納したり try-catch 構文の中で例外をスローしたりします。
        final boolean someError = false;
        if (someError) {
            throw new BloomMakerPreprocessException("Some error has occured.");
        }
        // 結果として、次のようなオブジェクトを返します。
        // {
        //   "foo": "bar",
        //   "accountContext": {
        //     "calendarId": "カレンダーID",
        //     "encoding": "エンコーディング",
        //     "userCd": "ユーザコード"
        //   },
        //   "currentDate": "ロケールに応じた現在日時"
        // }

        return result;
    }
}
